import fs from "fs";
import _ from "lodash";
import nodemailer from "nodemailer";
/**
 * # Handle mails With Nodemailer
 */
export default async function handleNodemailer(params) {
    var _a, _b;
    if (!process.env.DSQL_MAIL_HOST ||
        !process.env.DSQL_MAIL_EMAIL ||
        !process.env.DSQL_MAIL_PASSWORD) {
        return undefined;
    }
    let transporter = nodemailer.createTransport(Object.assign({ host: process.env.DSQL_MAIL_HOST, port: 465, secure: true, auth: {
            user: process.env.DSQL_MAIL_EMAIL,
            pass: process.env.DSQL_MAIL_PASSWORD,
        } }, params.options));
    const sender = (() => {
        var _a;
        if ((_a = params.alias) === null || _a === void 0 ? void 0 : _a.match(/support/i))
            return process.env.DSQL_MAIL_EMAIL;
        return process.env.DSQL_MAIL_EMAIL;
    })();
    const mailRootPath = process.env.DSQL_MAIL_ROOT || "./email/index.html";
    let mailRoot = fs.existsSync(mailRootPath)
        ? fs.readFileSync(mailRootPath, "utf8")
        : undefined;
    let finalHtml = mailRoot
        ? mailRoot
            .replace(/{{email_body}}/, ((_a = params.html) === null || _a === void 0 ? void 0 : _a.toString()) || "")
            .replace(/{{issue_date}}/, Date().substring(0, 24))
        : (_b = params.html) === null || _b === void 0 ? void 0 : _b.toString();
    try {
        let mailObject = {};
        mailObject["from"] = `"${params.senderName || "Datasquirel"}" <${sender}>`;
        mailObject["sender"] = sender;
        if (params.alias)
            mailObject["replyTo"] = sender;
        mailObject["to"] = params.to;
        mailObject["subject"] = params.subject;
        mailObject["text"] = params.text;
        mailObject["html"] = finalHtml;
        let info = await transporter.sendMail(Object.assign(Object.assign({}, _.omit(mailObject, ["alias", "senderName", "options"])), mailObject));
        return info;
    }
    catch (error) {
        console.log("ERROR in handleNodemailer Function =>", error.message);
    }
    return undefined;
}
