import serverError from "./serverError";
import parseDbResults from "./parseDbResults";
import connDbHandler from "../../utils/db/conn-db-handler";
/**
 * # Read Only Db Handler with Varaibles
 * @returns
 */
export default async function varReadOnlyDatabaseDbHandler({ queryString, queryValuesArray, tableSchema, forceLocal, }) {
    var _a;
    let results;
    const DB_CONN = forceLocal
        ? global.DSQL_DB_CONN
        : global.DSQL_READ_ONLY_DB_CONN || global.DSQL_DB_CONN;
    try {
        results = await connDbHandler(DB_CONN, queryString, queryValuesArray);
    }
    catch (error) {
        serverError({
            component: "varReadOnlyDatabaseDbHandler",
            message: error.message,
            noMail: true,
        });
        (_a = global.ERROR_CALLBACK) === null || _a === void 0 ? void 0 : _a.call(global, `VarReadOnlyDB Handler Error`, error);
        return error.message;
    }
    finally {
        DB_CONN === null || DB_CONN === void 0 ? void 0 : DB_CONN.end();
    }
    if (results) {
        const unparsedResults = results;
        const parsedResults = await parseDbResults({
            unparsedResults: unparsedResults,
            tableSchema: tableSchema,
        });
        return parsedResults;
    }
    else {
        return null;
    }
}
